require('isomorphic-fetch');
require('isomorphic-form-data');
const {
 searchItems,
 getItem,
 unprotectItem,
 removeItem
} = require("@esri/arcgis-rest-items");
const { UserSession } = require('@esri/arcgis-rest-auth');
const { getSelf } = require("@esri/arcgis-rest-request");
const chalk = require('chalk');
const requestOptions = {
 portal: 'https://www.arcgis.com'
};

let username = process.argv[2];
let password = process.argv[3];

requestOptions.authentication = new UserSession({
  clientId: 'arcgisonline',
  username: username,
  password: password,
  portal: `https://www.arcgis.com/sharing/rest`
});

function ownerViaRequestOptions(item, requestOptions) {
  return new Promise((resolve, request) => {
    setTimeout(() => {
      let color = 'green';
      if (item.owner !== requestOptions.owner) {
        color = 'red';
      }
      console.log(chalk[color](`REQUESTOPTIONS: Item Owner ${item.owner} and requestOptions.owner ${requestOptions.owner}`));
      resolve();
    });
  });
}

function ownerViaParam(item, owner, requestOptions) {
  return new Promise((resolve, request) => {
    setTimeout(() => {
      let color = 'green';
      if (item.owner !== owner) {
        color = 'red';
      }
      console.log(chalk[color](`PARAM: Item Owner ${item.owner} and owner param ${owner}`));
      resolve();
    });
  });
}


getSelf(requestOptions)
  .then((portalResponse) => {
    // issue a search request for a mess of Items not owned by this user
    let currentUser = requestOptions.authentication.username
    let query = `water -owner: ${currentUser}`;
    console.log(`Query: ${query}`);
    return searchItems({
        q: query,
        start: 1,
        num: 20
      }, requestOptions)
  })
  .then((response) => {
    console.log(`Got ${response.results.length} results back...`);
    // iterate the response, creating an array of promises w/ requestOptions w/ the owner on each...
    let promises = response.results.map((itm) => {
      requestOptions.owner = itm.owner;
      return ownerViaRequestOptions(itm, requestOptions);
    });
    let promisesWithParam = response.results.map((itm) => {
      return ownerViaParam(itm, itm.owner, requestOptions);
    });
    return Promise.all(promises.concat(promisesWithParam));
  })
  .catch((err) => {
    console.log(`Err ${err}`);
  })
