/**
 * Domain Service Wrapper
 */
 const {
  getItem
} = require("@esri/arcgis-rest-items");

const async = require('async');
const concurrency = 2;
module.exports = {

  isGuid (stringToTest) {
    if (stringToTest[0] === '{') {
      stringToTest = stringToTest.substring(1, stringToTest.length - 1);
    }
    var regexGuid = /^(\{){0,1}[0-9a-fA-F]{8}[-]?[0-9a-fA-F]{4}[-]?[0-9a-fA-F]{4}[-]?[0-9a-fA-F]{4}[-]?[0-9a-fA-F]{12}(\}){0,1}$/gi;
    return regexGuid.test(stringToTest);
  },

  getBySiteId (id, env) {
    if (env === 'prod'){
      env = '';
    }
    let url = `https://hub${env}.arcgis.com/utilities/domains?siteId=${id}`;

    return fetch(url)
      .then((response) => {
        return response.json();
      })
      .catch((err) => {
        console.log(`Error fetching domain from ${url}`);
      });
  },

  getByOrgId (id, env) {
    if (env === 'prod'){
      env = '';
    }
    let url = `https://hub${env}.arcgis.com/utilities/domains?orgId=${id}&limit=5000`;

    return fetch(url)
      .then((response) => {
        return response.json();
      })
      .catch((err) => {
        console.log(`Error fetching domain from ${url}`);
      });
  },

  remove (id, token, env) {
    if (env === 'prod'){
      env = '';
    }
    let options = {
      method: 'DELETE',
      headers: {
        'Content-Type': 'application/json',
        Authorization: token
      }
    };
    let url = `https://hub${env}.arcgis.com/utilities/domains/${id}`;
    return fetch(url, options)
      .then((response) => {
        return response.json();
      });
  },

  /**
   * Remove orphaned domain entries
   */
  removeOrphans (orgId, requestOptions) {
    console.log(`Env ${requestOptions.env}`);
    // get all entries for the org...
    return this.getByOrgId(orgId, requestOptions.env)
      .then((orgDomains) => {
        // now we want to see if this site still exists...
        // if not, we want to remove the entry
        let processQueue = async.queue((task, cb) => {
          // call remove and let it do it's work
          this.handleDomain(task, requestOptions)
            .then((result) => {
              cb();
            }, concurrency);
        });
         processQueue.push(orgDomains);
         return new Promise((resolve, reject) => {
           processQueue.drain = function () {
             console.log('Done processing all domains');
             resolve({success:true});
           }
         });
      })

  },

  handleDomain (domain, requestOptions) {
    let token = requestOptions.authentication.token;
    if (this.isGuid(domain.siteId)) {
      // it's an item...
      return getItem(domain.siteId, requestOptions)
      .then((response) => {
        let prms;
        if (response.error) {
          if (response.error.code === 400) {
            console.log(`No site found for domain ${domain.hostname} - removing domain entry`);
            prms = this.remove(domain.id, token, requestOptions.env)
            .then((response) => {
              console.log(`Removed domain entry for ${domain.hostname}`);
              return response;
            });
            //prms = Promise.resolve({success:true});
          } else {
            console.log(`Error fetching site ${response.error.message}`);
            prms = Promise.resolve({success:true});
          }
        } else {
          // this domain has an item, so all is well...
          console.log(`Found site for domain ${domain.hostname}`);
          prms = Promise.resolve({success:true});
        }
        return prms;
      }, (err) => {
        console.log(`No site found for domain ${domain.hostname} - removing domain entry`);
        return this.remove(domain.id, token, requestOptions.env)
        .then((response) => {
          console.log(`Removed domain entry for ${domain.hostname}`);
          return response;
        });
        //return Promise.resolve({success:true});
      })
      .catch((err) => {
        console.log(`Error ${err}`);
      })
    } else {
      // this is hosted on hub... we leave it alone
      // console.log(`${domain.hostname} site is not an item. Skipping...`);
      return Promise.resolve({success:true});
    }
  }
}
