/**
 * Module focused on purging groups
 */
 const {
  searchGroups,
  getGroup,
  unprotectGroup,
  removeGroup
  } = require("@esri/arcgis-rest-groups");

 const async = require('async');
 const chalk = require('chalk');
 const concurrency = 4;
 module.exports = {
   /**
    * Unprotect and remove the group
    */
   handleGroup: function (group, requestOptions) {
     // Since protected flag is returned incorrectly in groupSearch results...
     // we need to get the group
     return getGroup(group.id, requestOptions)
       .then((response) => {
         group = response;
         console.log(`Removing group ${group.title} - ${group.id} - ${group.protected}`);
         let prms;
         if (group.protected) {
           // unprotect... and return the group
           prms = unprotectGroup({id: group.id, ...requestOptions})
             .then((result) => {
               console.log(`Unprotected Group: ${group.id}`);
               return group;
             });
         } else {
           prms = Promise.resolve(group);
         }
         return prms;
       })
       .then((group) => {
         // console.log(`Removing Group: ${group.id}`);
         // return Promise.resolve({success:true}); //
         return removeGroup({
           id: group.id,
           ...requestOptions
         });
       })
       .then((result) => {
         if (result.success) {
           console.log(`Removed Group: ${group.id}`);
         }
       })
       .catch((err) => {
         console.error(chalk.red(`Error handling group ${err}`));
         return;
       });
   },
   /**
    * Process groups, using queues so we dont ddos things
    */
   process: function (orgId, query, requestOptions) {
     const owner = requestOptions.authentication.username;
     const q = `owner: ${owner} AND ${query}`;
     return searchGroups({
       q,
       start: 1,
       num: 1
     }, requestOptions)
     .then((result) => {
       let promises = [];
       let step = 100;
       let start = 1;
       for (start; start < result.total; start = start + step){
         promises.push(searchGroups({q, start:start, num:100}, requestOptions));
       }
       // this will either resolve, with an array of results
       // or reject if any of the promises reject
       return Promise.all(promises);
     })
     .then((searchPromises) => {
       let allGroups = [];
       searchPromises.map((resultSet) => {
         if (resultSet.results.length) {
           allGroups = allGroups.concat(resultSet.results);
         }
       });
       // create the queue...
       let processQueue = async.queue((task, cb) => {
         // call remove and let it do it's work
         this.handleGroup(task, requestOptions)
           .then((result) => {
             cb();
           }, concurrency);
       });
       // and pump it full of entries...
       processQueue.push(allGroups);

       return new Promise((resolve, reject) => {
         processQueue.drain = function () {
           console.log('Done processing all groups');
           resolve({success:true});
         }
       });
     })
   }
}
