/**
 * Module that knows how to systematically remove an initiative, and all it's children, and groups etc
 *
 * In order to not DDOS the portal api, we use a queue to manage how much is happening at the same time
 */
 const {
  searchItems,
  getItem,
  getItemData,
  unprotectItem,
  removeItem
 } = require("@esri/arcgis-rest-items");
const _ = require('lodash');

module.exports = {
  remove: function (item, requestOptions) {
    console.log(`Removing ${item.id}...`);
    // get the /data
    return getItemData(item.id, requestOptions)
    .then((data) => {
      console.log(`Got data for ${item.id}`);
      let model = {
        item: item,
        data: data
      };
      // ok let's see what we've got in terms of linked instances...
      // to do this, lets merge the items arrays which may/maynot exist under every key in .values
      let linkedItemIds = [];
      _.forIn(model.data.values, (v, k) => {
        if (v && v['items'] && Array.isArray(v['items'])) {
          linkedItemIds = linkedItemIds.concat(v['items']);
        }
      });
      console.log(`Initiative ${model.item.id} has ${linkedItemIds.length} linked items`);
      let groupsToRemove = [];
      if (model.item.properties) {
        if (model.item.properties.groupId) {
          groupsToRemove.push(model.item.properties.groupId)
        }
        if (model.item.properties.openDataGroupId) {
          groupsToRemove.push(model.item.properties.openDataGroupId)
        }
        console.log(` and ${groupsToRemove.length} groups to remove`);
      }
    })
    .then(() => {
      // remove the initiative item...
      return getItem(item.id, requestOptions);
    })
    .then((response) => {
      item = response;
      let prms;
      if (item.protected) {
        // unprotect... and return the item
        // console.log(`Trying to unprotect ${item.title}`);
        prms = unprotectItem({
          id: item.id,
          owner: item.owner,
          ...requestOptions
        })
          .then((result) => {
            // console.log(`Unprotected ${item.title}`);
            return item;
          }, (err) => {
            console.log(`Error trying to unprotect ${item.title} ${err}`);
          })

      } else {
        prms = Promise.resolve(item);
      }
      return prms
    })
    .then((item) => {
      return removeItem({
        id: item.id,
        owner: item.owner,
        ...requestOptions
      });
    })
    .then((result) => {
      if (!result.success) {
        console.log(`Error removing ${item.id}`);
      }
      return result;
    })
    .catch((err) => {
      console.log(`Error removing item ${item.id}. ${err}`);
    })

  }
}
