/**
 * Module focused on purging items
 */
 const {
  searchItems,
  getItem,
  unprotectItem,
  removeItem
} = require("@esri/arcgis-rest-items");

 const async = require('async');
 const concurrency = 4;
 module.exports = {
   /**
    * Unprotect and remove the item
    */
   handleItem: function (item, requestOptions) {
     // double check...
     if (item.typeKeywords.includes('doNotDelete') || item.type === 'Hub Site Application') {
       return Promise.resolve({success:true});
     } else {
       console.log(`Removing ${item.type} :: ${item.owner} :: ${item.id}`);
     }
     // Since the .protected property is not returned in an item search... we need to refetch the item...
     return getItem(item.id, requestOptions)
       .then((response) => {
         item = response;
         let prms;
         if (item.protected) {
           // unprotect... and return the item
           // console.log(`Trying to unprotect ${item.title}`);
           prms = unprotectItem({
             id: item.id,
             owner: item.owner,
             ...requestOptions
           })
             .then((result) => {
               // console.log(`Unprotected ${item.title}`);
               return item;
             }, (err) => {
               console.log(`Error trying to unprotect ${item.title} ${err}`);
             })

         } else {
           prms = Promise.resolve(item);
         }
         return prms
       })
       .then((item) => {
         return removeItem({
           id: item.id,
           owner: item.owner,
           ...requestOptions
         });
       })
       .then((result) => {
         if (!result.success) {
           console.log(`Error removing ${item.id}`);
         }
         return result;
       })
       .catch((err) => {
         console.error(`Error handling item ${err}`);
         return;
       });
   },
   /**
    * Process items, using queues so we dont ddos things
    */
   process: function (orgId, query, requestOptions) {
     const owner = requestOptions.authentication.username;
     const q = `owner: ${owner} AND -type:"Hub Site Application" AND ${query}`;
     return searchItems({
       searchForm: {
         q,
         start: 1,
         num: 1
       },
       ...requestOptions
     })
     .then((result) => {
       let promises = [];
       let step = 100;
       let start = 1;
       for (start; start < result.total; start = start + step){
         promises.push(searchItems({searchForm: {q, start:start, num:100}, ...requestOptions}));
       }
       // this will either resolve, with an array of results
       // or reject if any of the promises reject
       return Promise.all(promises);
     })
     .then((searchPromises) => {
       let allGroups = [];
       searchPromises.map((resultSet) => {
         if (resultSet.results.length) {
           allGroups = allGroups.concat(resultSet.results);
         }
       });

       // create the queue...
       let processQueue = async.queue((task, cb) => {
         // call remove and let it do it's work
         this.handleItem(task, requestOptions)
           .then((result) => {
             cb();
           });
       }, concurrency);
       // and pump it full of entries...
       processQueue.push(allGroups);

       return new Promise((resolve, reject) => {
         processQueue.drain = function () {
           console.log('Done processing all items');
           resolve({success:true});
         }
       });
     })
   }
}
