/**
 * Purge-command
 * implementation for the hub purge command
 */
require('isomorphic-fetch');
require('isomorphic-form-data');
const chalk = require('chalk');
const Groups = require('./groups');
const Items = require('./items');
const Initiatives = require('./initiatives');
const Sites = require('./sites');
const Domains = require('./domains');
const HubUrls = require('./hub-urls');
const { UserSession } = require('@esri/arcgis-rest-auth');
const { getSelf } = require("@esri/arcgis-rest-request");

module.exports = {
  execute: async function (username, password, portalUrl, options = {}) {
    const requestOptions = {
     portal: `${portalUrl}/sharing/rest`,
     env: HubUrls.getEnvFromPortalUrl(portalUrl)
    };

    requestOptions.authentication = new UserSession({
      clientId: 'arcgisonline',
      username: username,
      password: password,
      portal: `${portalUrl}/sharing/rest`
    });

    // Get portal/self so we can get the user's orgid so we
    // can limit the search to crap in their org
    let orgId;

    try {
      const res = await getSelf(requestOptions);
      orgId = res.id;
      console.log(`Got portal ${orgId}`);

    } catch (err) {
      console.log(chalk.red(`Error authenticating with ${portalUrl}. Verify credentials.`));
      console.error(err);
      return
    }

    // only execute the promise chain if the authentication works
    try {
      const groupQuery = options.groupQuery || '-tags: doNotDelete';
      console.log(chalk.green(`Removing groups marked with ${groupQuery}...`));
      await Groups.process(orgId, groupQuery, requestOptions);

      const itemQuery = options.itemQuery || '-typekeywords: doNotDelete';
      console.log(chalk.green(`Removing non-site items marked with ${itemQuery}...`));
      await Items.process(orgId, itemQuery, requestOptions);

      if (!options.skipSite) {
        console.log(chalk.green(`Removing Site items not marked with typeKeyword: doNotDelete...`));
        await Sites.process(orgId, '-typekeywords: doNotDelete', requestOptions);
      }

      if (!options.skipDomain) {
        console.log(chalk.green(`Removing orphan domains...`));
        await Domains.removeOrphans(orgId, requestOptions);
      }

      // console.log(chalk.green(`Removing Initiative items not marked with typeKeyword: doNotDelete...`));
      // await Initiatives.process(orgId, requestOptions);

      console.log(chalk.green(`Done.`));
    } catch (err) {
      console.error(err);
    }
  }
}
