/**
 * Module focused on purging items
 */
const {
  searchItems,
  getItem,
  getItemData,
  unprotectItem,
  removeItem
} = require("@esri/arcgis-rest-items");

const Domains = require('./domains');
const HubIndex = require('./hubindex');

 const async = require('async');
 const concurrency = 4;

 module.exports = {
   /**
    * Unprotect and remove the item
    */
   handleSite: function (item, requestOptions) {
     if (item.typeKeywords.includes('doNotDelete')) {
       return Promise.resolve({success:true});
     } else {
       console.log(`Removing ${item.type} :: ${item.owner} :: ${item.id}`);
     }
     let token = requestOptions.authentication.token;
     let site = item;

     return getItem(item.id, requestOptions)
       .then((response) => {
         site = response;
         let prms;
         if (site.protected) {
           console.log(`Unprotecting site ${site.id}`);
           // unprotect... and return the item
           prms = unprotectItem(site.id, site.owner, requestOptions)
             .then((result) => {
               return site;
             });
         } else {
           prms = Promise.resolve(site);
         }
         return prms;
       })
       .then((site) => {
         // get the domain entries for this site
         return Domains.getBySiteId(site.id, requestOptions.env);
       })
       .then((domains) => {
         // remove domain entries
         let domainPromises = [];
         domains.map((domain) => {
           console.log(`Removing domain ${domain.hostname} for site ${site.id}`);
           domainPromises.push(Domains.remove(domain.id, token, requestOptions.env));
         });
         // make sure we have at least one promise in here...
         domainPromises.push(Promise.resolve({success:true}));
         return Promise.all(domainPromises);
       })
       .then((domainResuts) => {
         // get the /data
         console.log(`Getting data for ${site.id}`);
         return getItemData(site.id, requestOptions);
       })
       .then((itemData) => {
         // get the siteId from the data...
         let openDataSiteId = itemData.values.siteId;
         if (openDataSiteId) {
           console.log(`Removing hubindex for ${site.id}`);
           return HubIndex.remove(openDataSiteId, token, requestOptions.env);
         } else {
           return Promise.resolve();
         }
       })
       .then((indexRemoveResponse) => {
         return removeItem({id:site.id, owner:site.owner, ...requestOptions});
         // return Promise.resolve({success:true}); //
       })
       .then((result) => {
         if (result.success) {
           // console.log(`Removed ${item.id}`);
         }
       })
       .catch((err) => {
         console.error(`Error handling item ${err}`);
         return;
       });
   },
   /**
    * Process items, using queues so we dont ddos things
    */
   process: function (orgId, query, requestOptions) {
     const owner = requestOptions.authentication.username;
     const q = `owner: ${owner} AND type: "Hub Site Application" AND ${query}`;
     return searchItems({
       searchForm: {
         q,
         start: 1,
         num: 1
       },
       ...requestOptions
     })
     .then((result) => {
       let promises = [];
       let step = 100;
       let start = 1;
       for (start; start < result.total; start = start + step){
         promises.push(searchItems({searchForm: {q, start:start, num:100}, ...requestOptions}));
       }
       // this will either resolve, with an array of results
       // or reject if any of the promises reject
       return Promise.all(promises);
     })
     .then((searchPromises) => {
       let allGroups = [];
       searchPromises.map((resultSet) => {
         if (resultSet.results.length) {
           allGroups = allGroups.concat(resultSet.results);
         }
       });

       // create the queue...
       let processQueue = async.queue((task, cb) => {
         // call remove and let it do it's work
         this.handleSite(task, requestOptions)
           .then((result) => {
             cb();
           }, concurrency);
       });
       // and pump it full of entries...
       processQueue.push(allGroups);

       return new Promise((resolve, reject) => {
         processQueue.drain = function () {
           console.log('Done processing all items');
           resolve({success:true});
         }
       });
     })
   }
}
